'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

//=============================================================================
// Cecilia_GridInventory.js                                                             
//=============================================================================


/*:
*
* @author Kino
* @plugindesc A grid based inventory for RPGMakerMV
*
* @param Large Window X Position
* @desc X position of the window.
* @default 500
*
* @param Large Window Y Position
* @desc Y position of the window.
* @default 100
*
* @param Inventory Window Rows
* @desc Number of rows in the window.
* @default 5
*
* @param Inventory Window Columns
* @desc Number of columns in the window.
* @default 5
* 
* @param Lock Large Window Position
* @desc Lock the position of the window.
* @default T
*
* @param Empty Icon Number
* @desc The icon number to represent an empty inventory space.
* @default 16
* 
* @param Inventory System Window Colors
* @desc Set the window color using RGB Values for all windows used in the system.
* @default 40, 40, 150
* 
* @param Small Window X Position
* @desc X position of the small UI window
* @default 500
*
* @param Small Window Y Position
* @desc Y position of the small UI window
* @default 500
*
* @param Lock Small Window Position
* @desc Locks the position of the small window
* @default T
*
* @param Number Of Columns For Item Description
* @desc The number of columns used for item description
* @default 4
* 
* @param Inventory Title Name
* @desc The title used in the large UI window.
* @default Inventory
*
* @param Use Item Text
* @desc Text for using an item.
* @default Use Item
*
* @param Equip Weapon Text
* @desc Text for equipping a weapon.
* @default Equip Weapon
*
* @param Equip Armor Text
* @desc Text for equipping armor.
* @default Equip Armor
* 
* @param Swap Item Text
* @desc Text used for swapping an item.
* @default Swap Item
*
* @param Remove Item Text
* @desc Text used for removing an item.
* @default Remove Item
* 
* @param Regular Item Color
* @desc Color of Regular Items; uses color code numbers.
* @default 1
*
* @param Weapon Color
* @desc Color of the circle for weapons; uses color code numbers.
* @default 18
*
* @param Armor Color
* @desc Color of the circle for armor; uses color code numbers.
* @default 6
*
* @param Key Item Color
* @desc Color of the circle for key items; uses color code numbers.
* @default 27
* 
* @param Num Inventory Pages
* @desc Number of inventory pages.
* @default 4
*
* @param Disable Mini Window
* @desc Whether to keep the mini window active or not; (T/F) true or false.
* @default T
*
*
* @help
*
* Version 1.01 Beta
//=============================================================================
//  Introduction                                                            
//=============================================================================
* 
* This plugin is completely touched based at the moment. This is based off
* FF14's Grid Inventory system. As of right now, you have quite a few options
* in terms of grid size, and items. You can also have tabs based on your inven-
* tory size (multiple pages). It comes with a mini window to open the inventory.
*
//=============================================================================
// How To Use                                                             
//=============================================================================
*
* Simply click on the small inventory window to open up the large inventory.
* Click on an item icon to open the context menu.
* The context menu has 3 choices Use / Swap / Remove
* 
* Use Item uses the item on the character of your choice. It opens 
* up a menu with actor names.  Click on an actor name to use the 
* item on the actor. 
*
* Swap Item
* Click swap item, then to swap an item click on the item you want to swap, 
* then click on the position you want to swap to.
*
* Click remove item, if you want to remove the item from the inventory
* completely.
//=============================================================================
// Script Calls                                                             
//=============================================================================
*
* Cecilia.openInventory();
* Opens the inventory window.
*
* Cecilia.openSmallInventory();
* Opens the small inventory window.
*  
* Cecilia.closeInventory();
* Closes the inventory window.
*
* Cecilia.closeSmallInventory();
* Closes the small inventory.
* 
* These manage the large  and small inventory window, so you can set up 
* common events to handle opening or closing the inventory window.
* 
//=============================================================================
//  Contact Information
//=============================================================================
*
* Contact me via twitter: EISKino, or on the rpg maker forums.
* Username on forums: Kino.
*
* Forum Link: http://forums.rpgmakerweb.com/index.php?/profile/75879-kino/
* Website Link: http://endlessillusoft.com/
* Twitter Link: https://twitter.com/EISKino
* Patreon Link: https://www.patreon.com/EISKino
*
* Hope this plugin helps, and enjoy!
* --Kino
*/

var Cecilia = Cecilia || {};

(function ($) {

  var params = PluginManager.parameters("Cecilia_GridInventory");
  var InvSysParams = {
    xPositionLarge: Number(params['Large Window X Position']),
    yPositionLarge: Number(params['Large Window Y Position']),
    windowLargeRows: Number(params['Inventory Window Rows']),
    windowLargeColumns: Number(params['Inventory Window Columns']),
    windowLockLarge: String(params['Lock Large Window Position']),
    emptyIconNumber: Number(params['Empty Icon Number']),
    windowColor: String(params['Inventory System Window Colors']),
    xPositionSmall: Number(params['Small Window X Position']),
    yPositionSmall: Number(params['Small Window Y Position']),
    windowLockSmall: String(params['Lock Small Window Position']),
    numDescriptionColumns: Number(params['Number Of Columns For Item Description']),
    titleText: String(params['Inventory Title Name']),
    itemText: String(params['Use Item Text']),
    weaponText: String(params['Equip Weapon Text']),
    armorText: String(params['Equip Armor Text']),
    swapText: String(params['Swap Item Text']),
    removeText: String(params['Remove Item Text']),
    itemColor: Number(params['Regular Item Color']),
    weaponColor: Number(params['Weapon Color']),
    armorColor: Number(params['Armor Color']),
    keyItemColor: Number(params['Key Item Color']),
    numPages: Number(params['Num Inventory Pages']),
    smallWindowActive: String(params['Disable Mini Window'])
  };

  var Setup = function Setup() {
    'use strict';

    //=============================================================================
    // InventorySystemInitializer                                                             
    //=============================================================================

    function InventorySystemInitializer() {}

    InventorySystemInitializer.initialize = function () {
      this.setupSystems();
    };

    InventorySystemInitializer.setupSystems = function () {
      InventoryManager.initialize();
      InventoryRequester.initialize();
      console.log("%c Cecilia Grid Inventory ", "color:#82CAFA; background-color:#1569C7; border: 1px solid #82CAFA;");
    };

    //=============================================================================
    // InventoryManager                                                             
    //=============================================================================

    function InventoryManager() {}

    InventoryManager.initialize = function () {
      this._currentPageNumber = 1;
      this._currentItem = null;
      this._itemForUse = null;
      this.initializeInventory();
    };

    InventoryManager.initializeInventory = function () {
      var inventorySize = Inventory.pageSlots * Inventory.totalPages;
      //if(Inventory.inventory.length > 100)
      Inventory.inventory = new Array(inventorySize);
    };

    InventoryManager.setInventoryPage = function (pageNumber) {
      this._currentPageNumber = pageNumber;
    };

    InventoryManager.setCurrentInventoryItem = function (item) {
      this._currentItem = item;
    };

    InventoryManager.setItemForUse = function (item) {
      this._itemForUse = item;
    };

    InventoryManager.getInventoryPage = function () {
      var pageSlots = Inventory.pageSlots;
      var index = this._currentPageNumber;
      var inventoryPage = Inventory.inventory.slice(pageSlots * (index - 1), pageSlots * index);
      return inventoryPage;
    };

    InventoryManager.getCurrentInventoryItem = function () {
      return this._currentItem;
    };

    InventoryManager.getItemForUse = function () {
      return this._itemForUse;
    };

    InventoryManager.getInventoryPageByIndex = function (index) {
      var pageSlots = Inventory.pageSlots;
      var inventoryPage = Inventory.inventory.slice(pageSlots * (index - 1), pageSlots * index);
      return inventoryPage;
    };

    InventoryManager.getPageNumber = function () {
      return this._currentPageNumber;
    };

    InventoryManager.swapItemPlace = function (originalIndex, newIndex) {
      var temp = Inventory.inventory[originalIndex];
      Inventory.inventory[originalIndex] = undefined;
      Inventory.inventory[newIndex] = temp;
    };

    InventoryManager.extendInventory = function () {
      var arrayLength = Inventory.pageSlots;
      Inventory.pages.push(new Array(arrayLength));
    };

    InventoryManager.adjustInventory = function () {
      var newItems = null;
      if ($gameParty) {
        newItems = this.getItemsNotInInventory();
        var index = 0;
        while (newItems.length > 0) {
          this.clearItemsWithZeroLeft();
          if (Inventory.inventory[index] === undefined) Inventory.inventory[index] = newItems.removeFromFront();else index++;
        }
        this.clearItemsWithZeroLeft();
        newItems = null;
      }
    };

    InventoryManager.getItemsNotInInventory = function () {
      var sortedInv = Inventory.inventory.copySortedArray();
      var partyItems = $gameParty.allItems();
      sortedInv.forEach(function (element) {
        for (var i = 0; i < partyItems.length; i++) {
          if (element === partyItems[i]) partyItems.splice(i, 1);
        }
      });
      return partyItems;
    };

    InventoryManager.clearItemsWithZeroLeft = function () {
      for (var i = 0; i < Inventory.inventory.length; i++) {
        if ($gameParty.numItems(Inventory.inventory[i]) <= 0) {
          Inventory.inventory[i] = undefined;
        }
      }
    };

    InventoryManager.processInventoryUpdates = function () {
      var gamePartyItems = $gameParty.allItems();
      var inventoryList = Inventory.inventory;
      gamePartyItems.sort();
      inventoryList.sort();
    };

    //=============================================================================
    // InventoryRequester                                                             
    //=============================================================================

    function InventoryRequester() {}

    InventoryRequester.initialize = function () {
      this.update();
    };

    InventoryRequester.update = function () {
      this.requestInventoryModification();
      this.requestUpdate();
    };

    InventoryRequester.requestUpdate = function () {
      requestAnimationFrame(this.update.bind(this));
    };

    InventoryRequester.requestInventoryModification = function () {
      InventoryManager.adjustInventory();
    };

    InventoryRequester.requestItemInformationSave = function (item) {
      InventoryManager.setCurrentInventoryItem(item);
    };

    InventoryRequester.setUseItem = function (item) {
      InventoryManager.setItemForUse(item);
    };

    InventoryRequester.getUseItem = function (item) {
      return InventoryManager.getItemForUse();
    };

    InventoryRequester.requestItemInformation = function () {
      return InventoryManager.getCurrentInventoryItem();
    };

    InventoryRequester.requestActivateItemSwapping = function () {
      var inventoryWindow = Util.getWindowFromScene("inventoryLargeWindow");
      inventoryWindow.activateSwapping();
    };

    InventoryRequester.requestItemSwap = function (initialPosition, newPosition) {
      InventoryManager.swapItemPlace(initialPosition, newPosition);
    };

    InventoryRequester.requestRemoveItem = function () {
      if ($gameParty) {
        var item = InventoryManager.getCurrentInventoryItem();
        var itemAmount = $gameParty.numItems(item);
        $gameParty.gainItem(item, itemAmount * -1);
      }
    };

    InventoryRequester.requestInventoryExpansion = function () {
      InventoryManager.extendInventory();
    };

    InventoryRequester.requestItemUseForSelectedUser = function () {
      var item = InventoryManager.getItemForUse();
      var user = Util.getWindowFromScene("inventoryActorWindow")._item;
      if (typeof user !== 'undefined') {
        if ($gameParty.numItems(item) > 0) {
          if (user.canUse(item)) InventoryRequester.useItemOnUser(user, item);
          if (user.canEquip(item)) InventoryRequester.equipItemOnUser(user, item);
        }
      }
    };

    InventoryRequester.useItemOnUser = function (user, item) {
      user.useItem(item);
      var action = new Game_Action(user);
      action.setItemObject(item);
      action.apply(user);
      action.applyGlobal();
    };

    InventoryRequester.equipItemOnUser = function (user, item) {
      user.changeEquip(item.etypeId - 1, item);
    };

    InventoryRequester.requestSmallInventoryOpen = function () {
      var inventorySmallWindow = Util.getWindowFromScene("inventorySmallWindow");
      inventorySmallWindow.open();
    };

    InventoryRequester.requestSmallInventoryClose = function () {
      var inventorySmallWindow = Util.getWindowFromScene("inventorySmallWindow");
      inventorySmallWindow.close();
    };

    InventoryRequester.requestInventoryPageOpen = function (pageNumber) {
      var inventoryWindow = Util.getWindowFromScene("inventoryLargeWindow");
      InventoryManager.setInventoryPage(pageNumber);
      inventoryWindow.open();
    };

    InventoryRequester.requestItemInformationWindowOpen = function (x, y) {
      var inventoryItemInfoWindow = Util.getWindowFromScene("inventoryInformationWindow");
      inventoryItemInfoWindow.openAtMousePosition(x, y);
    };

    InventoryRequester.requestItemInformationWindowClose = function () {
      var inventoryItemInfoWindow = Util.getWindowFromScene("inventoryInformationWindow");
      inventoryItemInfoWindow.close();
    };

    InventoryRequester.requestMouseMenuOpen = function (x, y) {
      var inventoryMouseMenu = Util.getWindowFromScene("inventoryMouseMenu");
      var inventoryWindow = Util.getWindowFromScene("inventoryLargeWindow");
      if (typeof InventoryManager.getItemForUse() !== 'undefined') {
        inventoryWindow.setActive(false);
        inventoryMouseMenu.openAtMousePosition(x, y);
      }
    };

    InventoryRequester.requestItemUseWindow = function () {
      var itemUseWindow = Util.getWindowFromScene("inventoryActorWindow");
      itemUseWindow.open();
    };

    InventoryRequester.requestInventoryClose = function () {
      var inventoryWindow = Util.getWindowFromScene("inventoryLargeWindow");
      inventoryWindow.close();
    };

    //=============================================================================
    // Inventory                                                             
    //=============================================================================

    function Inventory() {}

    Inventory.totalPages = InvSysParams.numPages;
    Inventory.pageSlots = InvSysParams.windowLargeRows * InvSysParams.windowLargeColumns;
    Inventory.inventory = [];

    //=============================================================================
    // System StartUp                                                             
    //=============================================================================
    InventorySystemInitializer.initialize();

    //=============================================================================
    // Scene_Map                                                             
    //=============================================================================
    var SceneMap_start = Scene_Map.prototype.start;
    Scene_Map.prototype.start = function () {
      SceneMap_start.call(this);
      this._windowInvSmall.open();
    };

    var CLISceneMap_createAllWindows = Scene_Map.prototype.createAllWindows;
    Scene_Map.prototype.createAllWindows = function () {
      CLISceneMap_createAllWindows.call(this);
      this.createInventorySmallWindow();
      this.createInventoryLargeWindow();
      this.createInventoryInformationWindow();
      this.createInventoryMouseMenuWindow();
      this.createInventoryActorWindow();
    };

    Scene_Map.prototype.createInventorySmallWindow = function () {
      var x = InvSysParams.xPositionSmall;
      var y = InvSysParams.yPositionSmall;
      this._windowInvSmall = new Cecilia_WindowInvSmall(x, y, 200, 75);
      this.addCLIWindow("inventorySmallWindow", this._windowInvSmall);
    };

    Scene_Map.prototype.createInventoryLargeWindow = function () {
      var x = InvSysParams.xPositionLarge;
      var y = InvSysParams.yPositionLarge;
      this._windowInvLarge = new Cecilia_WindowInvLarge(x, y, 100, 250);
      this.addCLIWindow("inventoryLargeWindow", this._windowInvLarge);
    };

    Scene_Map.prototype.createInventoryInformationWindow = function () {
      var x = InvSysParams.xPositionLarge;
      var y = InvSysParams.yPositionLarge;
      this._windowInvInformation = new Cecilia_WindowItemInformation(x - 160, y, 125, 150);
      this.addCLIWindow("inventoryInformationWindow", this._windowInvInformation);
    };

    Scene_Map.prototype.createInventoryMouseMenuWindow = function () {
      this._invMouseMenu = new Cecilia_WindowMouseMenu(0, 0, 75, 100);
      this.addCLIWindow("inventoryMouseMenu", this._invMouseMenu);
    };

    Scene_Map.prototype.createInventoryActorWindow = function () {
      this._invActorWindow = new Cecilia_WindowActorItemChoice(Graphics.width / 2 - 50, Graphics.height / 2 - 80, 100, 100);
      this.addCLIWindow("inventoryActorWindow", this._invActorWindow);
    };

    var SceneMap_terminate = Scene_Map.prototype.terminate;
    Scene_Map.prototype.terminate = function () {
      SceneMap_terminate.call(this);
      this._windowInvLarge.close();
      this._windowInvSmall.close();
    };

    //=============================================================================
    // DataManager                                                             
    //=============================================================================
    var CLIGDataManager_makeSaveContents = DataManager.makeSaveContents;
    DataManager.makeSaveContents = function () {
      var contents = {};
      contents = CLIGDataManager_makeSaveContents.call(this);
      contents.inventory = Inventory.inventory;
      return contents;
    };

    var CLIGDataManager_extractSaveContents = DataManager.extractSaveContents;
    DataManager.extractSaveContents = function (contents) {
      CLIGDataManager_extractSaveContents.call(this, contents);
      Inventory.inventory = contents.inventory;
    };

    //=============================================================================
    // Cecilia_WindowInvSmall                                                             
    //=============================================================================
    function Cecilia_WindowInvSmall() {
      this.initialize.apply(this, arguments);
    }

    Cecilia_WindowInvSmall.prototype = Object.create(Cecilia_WindowBase.prototype);
    Cecilia_WindowInvSmall.prototype.constructor = Cecilia_WindowInvSmall;

    Cecilia_WindowInvSmall.prototype.initialize = function (x, y, width, height) {
      Cecilia_WindowBase.prototype.initialize.call(this, x, y, width, height);
      if (this.isDisabled()) this.openness = 0;else this.open();
      this.processWindowLock();
    };

    Cecilia_WindowInvSmall.prototype.processWindowLock = function () {
      if (/T/ig.test(InvSysParams.windowLockSmall)) this.lockWindow();
    };

    Cecilia_WindowInvSmall.prototype.setup = function () {
      this.setupWindowColor();
      this.setupClickableAreas();
    };

    Cecilia_WindowInvSmall.prototype.setupWindowColor = function () {
      var colors = InvSysParams.windowColor.split(", ");
      this.setWindowTone(Number(colors[0]), Number(colors[1]), Number(colors[2]));
    };

    Cecilia_WindowInvSmall.prototype.setupClickableAreas = function () {
      var contentWidth = 7 * InvSysParams.windowLargeColumns * Inventory.totalPages + this.standardPadding();
      var areaWidth = Math.ceil((contentWidth + 3) / Inventory.totalPages);
      this.createRow(contentWidth);
      for (var i = 0; i < Inventory.totalPages; i++) {
        this.createColumn(1, areaWidth, 7 * InvSysParams.windowLargeRows);
        this.bindAction(1, i + 1, InventoryRequester.requestInventoryPageOpen.bind(this, i + 1));
      }
    };

    Cecilia_WindowInvSmall.prototype.update = function () {
      Cecilia_WindowBase.prototype.update.call(this);
      this.drawInventoryRepresentation();
    };

    Cecilia_WindowInvSmall.prototype.drawInventoryRepresentation = function () {
      var column = null;
      var rowColumnLength = this.getColumnLength(0);
      for (var i = 0; i < rowColumnLength; i++) {
        column = this.getColumn(1, i + 1);
        this.drawItemCircle(column, i + 1);
      }
    };

    Cecilia_WindowInvSmall.prototype.drawItemCircle = function (column, pageIndex) {
      var radius = 3.5;
      var x = column.x + radius;
      var y = column.y + radius;
      var color = 0;
      for (var rowNum = 0; rowNum < InvSysParams.windowLargeRows; rowNum++) {
        var adjustedY = this.calculateCirclePositions(radius, y, rowNum);
        for (var columnNum = 0; columnNum < InvSysParams.windowLargeColumns; columnNum++) {
          var adjustedX = this.calculateCirclePositions(radius, x, columnNum);
          var index = InvSysParams.windowLargeColumns * rowNum + columnNum;
          var inventoryPage = this.getInventoryPage(pageIndex);
          if (typeof inventoryPage[index] !== 'undefined') {
            color = this.processItemColor(inventoryPage[index]);
            this.contents.drawCircle(adjustedX, adjustedY, radius, this.textColor(color));
          }
        }
      }
    };

    Cecilia_WindowInvSmall.prototype.calculateCirclePositions = function (radius, x, y) {
      return x + (1 + radius * 2) * y;
    };

    Cecilia_WindowInvSmall.prototype.getInventoryPage = function (index) {
      return InventoryManager.getInventoryPageByIndex(index);
    };

    Cecilia_WindowInvSmall.prototype.processItemColor = function (item) {
      if (this.isItemType('item', item)) {
        return InvSysParams.itemColor;
      } else if (this.isItemType('weapon', item)) {
        return InvSysParams.weaponColor;
      } else if (this.isItemType('armor', item)) {
        return InvSysParams.armorColor;
      } else if (this.isItemType('keyItem', item)) {
        return InvSysParams.keyItemColor;
      } else return 0;
    };

    Cecilia_WindowInvSmall.prototype.isItemType = function (string, item) {
      switch (string) {
        case 'item':
          return DataManager.isItem(item) && item.itypeId === 1;
        case 'weapon':
          return DataManager.isWeapon(item);
        case 'armor':
          return DataManager.isArmor(item);
        case 'keyItem':
          return DataManager.isItem(item) && item.itypeId === 2;
        default:
          return false;
      }
    };

    Cecilia_WindowInvSmall.prototype.open = function () {
      if (!this.isOpen() && !this.isDisabled()) {
        this._opening = true;
      }
      this._closing = false;
    };

    Cecilia_WindowInvSmall.prototype.isDisabled = function () {
      return (/F/ig.test(InvSysParams.smallWindowActive)
      );
    };

    //=============================================================================
    // Cecilia_WindowInvLarge                                                             
    //=============================================================================
    function Cecilia_WindowInvLarge() {
      this.initialize.apply(this, arguments);
    }

    Cecilia_WindowInvLarge.prototype = Object.create(Cecilia_WindowBase.prototype);
    Cecilia_WindowInvLarge.prototype.constructor = Cecilia_WindowInvLarge;

    Cecilia_WindowInvLarge.prototype.initialize = function (x, y, width, height) {
      this.rows = InvSysParams.windowLargeRows;
      this.columns = InvSysParams.windowLargeColumns;
      this.rowWidth = this.columns * 45;
      this.columnHeight = 50;
      Cecilia_WindowBase.prototype.initialize.call(this, x, y, width, height);
      this.setZIndex(1);
      this._swapping = false;
      this._swapStartPosition = 0;
      this._columnIndexNumber = undefined;
      this.openness = 0;
      this.processWindowLock();
    };

    Cecilia_WindowInvLarge.prototype.processWindowLock = function () {
      if (/T/ig.test(InvSysParams.windowLockLarge)) this.lockWindow();
    };

    Cecilia_WindowInvLarge.prototype.setup = function () {
      this.setWindowColor();
      this.setupTitleArea();
      this.setupItemWindowTabs();
      this.setupItemGrid();
      this.setupGoldArea();
      this.setupTabActions();
      this.setupGridActionsAndColumnNumbers();
    };

    Cecilia_WindowInvLarge.prototype.setWindowColor = function () {
      var colors = InvSysParams.windowColor.split(", ");
      this.setWindowTone(Number(colors[0]), Number(colors[1]), Number(colors[2]));
    };

    Cecilia_WindowInvLarge.prototype.setupTitleArea = function () {
      this.createRow(this.rowWidth);
      this.createColumn(1, (this.columns - 1) * 45, this.columnHeight);
      this.createColumn(1, 35, 40);
      this.lockColumn(1, 1);
      this.bindAction(1, 2, this.close.bind(this));
    };

    Cecilia_WindowInvLarge.prototype.setupItemWindowTabs = function () {
      var tabs = InvSysParams.numPages;
      var tabWidth = Math.floor(this.rowWidth / this.columns);
      if (tabs > 1) {
        this.createRow(this.rowWidth);
        for (var x = 0; x < tabs; x++) {
          var row = this._rows.length;
          this.createColumn(row, tabWidth, this.columnHeight - 25);
        }
      }
    };

    Cecilia_WindowInvLarge.prototype.setupItemGrid = function () {
      for (var i = 0; i < this.rows; i++) {
        this.createRow(this.rowWidth);
        for (var x = 0; x < this.columns; x++) {
          var row = this._rows.length;
          this.createColumn(row, 45, this.columnHeight);
        }
      }
    };

    Cecilia_WindowInvLarge.prototype.setupGoldArea = function () {
      var rowNum = this._rows.length + 1;
      this.createRow(this.rowWidth);
      this.createColumn(rowNum, this.rowWidth, 30);
      this.lockColumn(rowNum, 1);
    };

    Cecilia_WindowInvLarge.prototype.setupTabActions = function () {
      var tabs = InvSysParams.numPages;
      if (tabs > 1) {
        var rowNum = this._rows.length - (this.rows + 1);
        for (var i = 1; i < tabs + 1; i++) {
          this.bindAction(rowNum, i, InventoryRequester.requestInventoryPageOpen.bind(this, i));
        }
      }
    };

    Cecilia_WindowInvLarge.prototype.setupGridActionsAndColumnNumbers = function () {
      var column = null;
      for (var i = 0; i < this.rows; i++) {
        for (var x = 0; x < this.columns; x++) {
          var rowNum = this._rows.length - this.rows;
          this.bindAction(i + rowNum, x + 1, this.processInventoryItem.bind(this, x, i));
          column = this.getColumn(i + rowNum, x + 1);
          column.indexNumber = InvSysParams.windowLargeColumns * i + x;
        }
      }
    };

    Cecilia_WindowInvLarge.prototype.update = function () {
      Cecilia_WindowBase.prototype.update.call(this);
      this.processSwapping();
      this.processItemInformation();
      this.drawAllItems();
    };

    Cecilia_WindowInvLarge.prototype.processSwapping = function () {
      var pageNumber = 0;
      if (this._swapping) {
        pageNumber = InventoryManager.getPageNumber();
        if (this._columnIndexNumber !== undefined) {
          if (this.calculateItemPosition(this._columnIndexNumber) !== this._swapStartPosition) {
            InventoryRequester.requestItemSwap(this._swapStartPosition, this.calculateItemPosition(this._columnIndexNumber));
            this.deactivateSwapping();
          }
        }
      }
    };

    Cecilia_WindowInvLarge.prototype.touchedColumn = function (column) {
      var distanceX = this.canvasToLocalX(this._touchedPoint.x);
      var distanceY = this.canvasToLocalY(this._touchedPoint.y);
      if (this.isTouchedInsideColumnFrame(distanceX, distanceY, column.width, column.height, column.x, column.y)) {
        this._touchedColumn = column;
        this._columnIndexNumber = this._touchedColumn.indexNumber;
        this._columnTouched = true;
      } else this._columnTouched = false;
    };

    Cecilia_WindowInvLarge.prototype.processItemInformation = function () {
      if (this._mousedOverColumn !== null && this._mousedOverColumn !== undefined && this._touchedColumn === null) {
        var inventoryPage = this.getInventoryPage();
        var index = this.calculateItemPosition(this._mousedOverColumn.indexNumber);
        var item = inventoryPage[index];
        var column = this._mousedOverColumn;
        if (item !== undefined) {
          if (item !== InventoryRequester.requestItemInformation()) InventoryRequester.requestItemInformationSave(item);
          InventoryRequester.requestItemInformationWindowOpen(this.x + column.x - column.width - 5, this.y + column.y + column.height + 7);
        } else InventoryRequester.requestItemInformationWindowClose();
      }
    };

    Cecilia_WindowInvLarge.prototype.activateSwapping = function () {
      this._swapping = true;
      this._swapStartPosition = this.calculateItemPosition(this.getTouchedColumnIndex());
    };

    Cecilia_WindowInvLarge.prototype.deactivateSwapping = function () {
      this._swapping = false;
    };

    Cecilia_WindowInvLarge.prototype.calculateItemPosition = function (indexNumber) {
      return indexNumber + (InventoryManager.getPageNumber() - 1) * Inventory.pageSlots;
    };

    Cecilia_WindowInvLarge.prototype.getTouchedColumnIndex = function () {
      return this._columnIndexNumber;
    };

    Cecilia_WindowInvLarge.prototype.drawAllItems = function () {
      this.drawTitle();
      this.drawClose();
      this.drawTabNumbers();
      this.drawInventoryPageItems();
      this.drawCurrency();
    };

    Cecilia_WindowInvLarge.prototype.drawTitle = function () {
      this.contents.fontSize = 20;
      var column = this.getColumn(1, 1);
      var textWidth = this.textWidth(InvSysParams.titleText);
      var midPoint = column.width / 2 - textWidth / 2;
      this.drawText(InvSysParams.titleText, midPoint, column.y, column.width);
      this.resetFontSettings();
    };

    Cecilia_WindowInvLarge.prototype.drawClose = function () {
      this.contents.fontSize = 20;
      var column = this.getColumn(1, 2);
      var textWidth = this.textWidth('X');
      var midPoint = column.x + column.width / 2 - textWidth / 2;
      this.drawText('X', midPoint, column.y, column.width);
      this.resetFontSettings();
    };

    Cecilia_WindowInvLarge.prototype.drawInventoryPageItems = function () {
      // let column = null;
      var index = 0;
      var inventoryPage = this.getInventoryPage();
      var rowNum = this._rows.length - this.rows;
      for (var i = 0; i < this.rows; i++) {
        for (var x = 0; x < this.columns; x++) {
          var column = this.getColumn(i + rowNum, x + 1);
          index = InvSysParams.windowLargeColumns * i + x;
          if (typeof inventoryPage[index] !== 'undefined') {
            this.drawInventoryIcon(inventoryPage[index], column.x + 6, column.y + 10);
            this.drawItemAmount(inventoryPage[index], column);
          } else {
            var iconNumber = InvSysParams.emptyIconNumber;
            this.drawIcon(iconNumber, column.x + 6, column.y + 10);
          }
        }
      }
    };

    Cecilia_WindowInvLarge.prototype.drawInventoryIcon = function (item, x, y) {
      var icon = item.iconIndex;
      this.drawIcon(icon, x, y);
    };

    Cecilia_WindowInvLarge.prototype.drawItemAmount = function (item, column) {
      var itemNumber = $gameParty.numItems(item);
      var textPosition = column.x + column.width - this.textWidth(itemNumber.toString());
      this.drawText(itemNumber, textPosition, column.y + 20);
    };

    Cecilia_WindowInvLarge.prototype.drawTabNumbers = function () {
      this.contents.fontSize = 18;
      var tabs = InvSysParams.numPages;
      if (tabs > 1) {
        var rowNum = this._rows.length - (this.rows + 1);
        for (var x = 0; x < tabs; x++) {
          var column = this.getColumn(rowNum, x + 1);
          var midPoint = column.x + column.width / 2 - this.textWidth("A") / 2;
          this.drawText(x + 1, midPoint, column.y - 5, column.width);
        }
      }
      this.resetFontSettings();
    };

    Cecilia_WindowInvLarge.prototype.drawCurrency = function () {
      var rowNum = this._rows.length;
      var column = this.getColumn(rowNum, 1);
      this.drawCurrencyValue($gameParty.gold(), TextManager.currencyUnit, column.x, column.y, column.width);
    };

    Cecilia_WindowInvLarge.prototype.getInventoryPage = function () {
      return InventoryManager.getInventoryPage();
    };

    Cecilia_WindowInvLarge.prototype.processInventoryItem = function (x, y) {
      var rowNum = this._rows.length - this.rows;
      var inventoryPage = this.getInventoryPage();
      var index = InvSysParams.windowLargeColumns * y + x;
      var item = inventoryPage[index];
      var column = this.getColumn(y + rowNum, x + 1);
      var xPosition = this.x + column.x + column.width - 5;
      var yPosition = this.y + column.y + column.height - 5;
      InventoryRequester.setUseItem(item);
      InventoryRequester.requestMouseMenuOpen(xPosition, yPosition);
    };

    //=============================================================================
    //  Cecilia_WindowItemInformation                                                            
    //=============================================================================
    function Cecilia_WindowItemInformation() {
      this.initialize.apply(this, arguments);
    }

    Cecilia_WindowItemInformation.prototype = Object.create(Cecilia_WindowBase.prototype);
    Cecilia_WindowItemInformation.prototype.constructor = Cecilia_WindowItemInformation;

    Cecilia_WindowItemInformation.prototype.initialize = function (x, y, width, height) {
      Cecilia_WindowBase.prototype.initialize.call(this, x, y, width, height);
      this.setZIndex(1);
      this.openness = 0;
      this.lockWindow();
    };

    Cecilia_WindowItemInformation.prototype.setup = function () {
      this.setWindowColor();
      this.setupItemInformationColumns();
    };

    Cecilia_WindowItemInformation.prototype.setWindowColor = function () {
      var colors = InvSysParams.windowColor.split(", ");
      this.setWindowTone(Number(colors[0]), Number(colors[1]), Number(colors[2]));
    };

    Cecilia_WindowItemInformation.prototype.setupItemInformationColumns = function () {
      var rowWidth = this.width;
      this.createRow(rowWidth);
      this.createColumn(1, rowWidth, 30);
      this.lockColumn(1, 1);
      this.setupDescriptionColumns(InvSysParams.numDescriptionColumns);
    };

    Cecilia_WindowItemInformation.prototype.setupDescriptionColumns = function (numColumns) {
      for (var i = 0; i < numColumns; i++) {
        this.createRow(this.width);
        this.createColumn(i + 2, this.width, 25);
        this.lockColumn(i + 2, 1);
      }
    };

    Cecilia_WindowItemInformation.prototype.update = function () {
      Cecilia_WindowBase.prototype.update.call(this);
      this.processDrawingItemInformation();
    };

    Cecilia_WindowItemInformation.prototype.processDrawingItemInformation = function () {
      var item = InventoryRequester.requestItemInformation();
      if (item !== undefined && item !== null) this.drawItemInformation(item);
    };

    Cecilia_WindowItemInformation.prototype.drawItemInformation = function (item) {
      this.drawItemName(item);
      this.drawItemDescription(item);
    };

    Cecilia_WindowItemInformation.prototype.drawItemName = function (item) {
      var column = this.getColumn(1, 1);
      this.contents.fontSize = 18;
      var midPoint = (column.x + column.width) / 2 - this.textWidth(item.name) / 2;
      this.drawText(item.name, midPoint, column.y, column.width);
      this.resetFontSettings();
    };

    Cecilia_WindowItemInformation.prototype.drawItemDescription = function (item) {
      var text = item.description;
      var textArray = this.createColumnSubStrings(text);
      this.clearLeadingWhiteSpace(textArray);
      for (var i = 0; i < textArray.length; i++) {
        this.drawInColumn(textArray[i], 2 + i);
      }
      this.resetFontSettings();
    };

    Cecilia_WindowItemInformation.prototype.createColumnSubStrings = function (text) {
      var rowText = [];
      var startIndex = 0;
      var count = 1;
      var columnWidth = this.getColumn(2, 1).width;
      while (count !== text.length) {
        this.contents.fontSize = 12;
        if (this.textWidth(text.substring(startIndex, count++)) > columnWidth) {
          while (this.textWidth(text.substring(startIndex, count)) > columnWidth) {
            count--;
          }rowText.push(text.substring(startIndex, count));
          startIndex = count;count = count + 1;
        } else if (count === text.length) {
          rowText.push(text.substring(startIndex, count));
        }
      }
      return rowText;
    };

    Cecilia_WindowItemInformation.prototype.clearLeadingWhiteSpace = function (array) {
      array.forEach(function (substring, index) {
        array[index] = substring.trimLeft();
      });
    };

    Cecilia_WindowItemInformation.prototype.drawInColumn = function (text, rowIndex) {
      if (rowIndex < this.getNumRows()) {
        var column = this.getColumn(rowIndex, 1);
        this.drawText(text, column.x, column.y, column.width);
      }
    };

    Cecilia_WindowItemInformation.prototype.openAtMousePosition = function (x, y) {
      //this.moveS(x, y);
      this.open();
    };

    //=============================================================================
    // Cecilia_WindowMouseMenu                                                             
    //=============================================================================

    function Cecilia_WindowMouseMenu() {
      this.initialize.apply(this, arguments);
    }

    Cecilia_WindowMouseMenu.prototype = Object.create(Cecilia_WindowBase.prototype);
    Cecilia_WindowMouseMenu.prototype.constructor = Cecilia_WindowMouseMenu;

    Cecilia_WindowMouseMenu.prototype.initialize = function (x, y, width, height) {
      Cecilia_WindowBase.prototype.initialize.call(this, x, y, width, height);
      this.setZIndex(2);
      this._mousedOverOnce = false;
      this.openness = 0;
    };

    Cecilia_WindowMouseMenu.prototype.setup = function () {
      this.setupWindowColor();
      this.setupCommandButtons();
      this.setupCommandActions();
    };

    Cecilia_WindowMouseMenu.prototype.setupWindowColor = function () {
      var colors = InvSysParams.windowColor.split(", ");
      this.setWindowTone(Number(colors[0]), Number(colors[1]), Number(colors[2]));
    };

    Cecilia_WindowMouseMenu.prototype.setupCommandButtons = function () {
      for (var i = 1; i < 4; i++) {
        this.createRow(this.width);
        this.createColumn(i, this.width, 30);
      }
    };

    Cecilia_WindowMouseMenu.prototype.setupCommandActions = function () {
      this.bindAction(1, 1, InventoryRequester.requestItemUseWindow.bind(this));
      this.bindAction(2, 1, InventoryRequester.requestActivateItemSwapping.bind(this));
      this.bindAction(3, 1, InventoryRequester.requestRemoveItem.bind(this));
    };

    Cecilia_WindowMouseMenu.prototype.update = function () {
      Cecilia_WindowBase.prototype.update.call(this);
      this.processWindowClosing();
      this.drawCommandButtons();
    };

    Cecilia_WindowMouseMenu.prototype.processWindowClosing = function () {
      if (this.isMousedOver()) {
        this._mousedOverOnce = true;
      }
      if (Util.getWindowFromScene("inventoryLargeWindow").isClosed()) this.close();
      if (!this.isMousedOver() && this._mousedOverOnce === true) {
        this.close();
        this._mousedOverOnce = false;
      }
      if (typeof InventoryRequester.getUseItem() === 'undefined') this.close();
    };

    Cecilia_WindowMouseMenu.prototype.drawCommandButtons = function () {
      this.contents.fontSize = 18;
      var textCommands = [InvSysParams.itemText, InvSysParams.swapText, InvSysParams.removeText];
      var item = InventoryRequester.getUseItem();
      for (var i = 0; i < textCommands.length; i++) {
        var column = this.getColumn(i + 1, 1);
        var text = i === 0 && !DataManager.isItem(item) ? this.processItemText(item) : textCommands[i];
        this.drawText(text, column.x, column.y, column.width);
      }
      this.resetFontSettings();
    };

    Cecilia_WindowMouseMenu.prototype.processItemText = function (item) {
      var text = "";
      if (DataManager.isWeapon(item)) text = InvSysParams.weaponText;
      if (DataManager.isArmor(item)) text = InvSysParams.armorText;
      return text;
    };

    Cecilia_WindowMouseMenu.prototype.openAtMousePosition = function (x, y) {
      this.moveS(x, y);
      this.open();
    };

    //=============================================================================
    // Cecilia_WindowActorItemChoice                                                             
    //=============================================================================

    function Cecilia_WindowActorItemChoice() {
      this.initialize.apply(this, arguments);
    }

    Cecilia_WindowActorItemChoice.prototype = Object.create(Cecilia_WindowSelectable.prototype);
    Cecilia_WindowActorItemChoice.prototype.constructor = Cecilia_WindowActorItemChoice;

    Cecilia_WindowActorItemChoice.prototype.initialize = function (x, y, width, height) {
      Cecilia_WindowSelectable.prototype.initialize.call(this, x, y, width, height);
      this.openness = 0;
      this.setZIndex(3);
    };

    Cecilia_WindowActorItemChoice.prototype.setup = function () {
      this.setupWindowColor();
      this.setupWindowRows();
    };

    Cecilia_WindowActorItemChoice.prototype.setupWindowColor = function () {
      var colors = InvSysParams.windowColor.split(", ");
      this.setWindowTone(Number(colors[0]), Number(colors[1]), Number(colors[2]));
    };

    Cecilia_WindowActorItemChoice.prototype.setupWindowRows = function () {
      for (var i = 0; i < 4; i++) {
        this.createRow(100);
        this.createColumn(i + 1, 100, 25);
        this.bindAction(i + 1, 1, InventoryRequester.requestItemUseForSelectedUser.bind(this));
      }
    };

    Cecilia_WindowActorItemChoice.prototype.setVariables = function () {
      this.setList($gameParty.allMembers());
      this.setMaxItems($gameParty.size());
      this.setMaxItemsPerPage(4);
    };

    Cecilia_WindowActorItemChoice.prototype.update = function () {
      Cecilia_WindowSelectable.prototype.update.call(this);
      this.processWindowClosing();
    };

    Cecilia_WindowActorItemChoice.prototype.processWindowClosing = function () {
      if (this.isMousedOver()) {
        this._mousedOverOnce = true;
      }
      if (Util.getWindowFromScene("inventoryLargeWindow").isClosed()) this.close();
      if (!this.isMousedOver() && this._mousedOverOnce === true) {
        this.close();
        this._mousedOverOnce = false;
      }
    };

    Cecilia_WindowActorItemChoice.prototype.drawItem = function (index, column) {
      var actor = $gameParty.members()[index];
      if (typeof actor !== 'undefined') {
        var xPosition = (column.x + column.width) / 2 - this.textWidth(actor.name()) / 2;
        this.drawText(actor.name(), xPosition, column.y - 5, column.width);
      }
    };

    Cecilia_WindowActorItemChoice.prototype.close = function () {
      Cecilia_WindowSelectable.prototype.close.call(this);
      InventoryRequester.setUseItem(undefined);
    };

    //=============================================================================
    // Utilities                                                             
    //=============================================================================

    var Util = function () {
      function Util() {
        _classCallCheck(this, Util);
      }

      _createClass(Util, null, [{
        key: 'getWindowFromScene',
        value: function getWindowFromScene(windowName) {
          var scene = SceneManager.getCurrentSceneCLI();
          var sceneWindow = scene.getWindowFromCLIList(windowName);
          return sceneWindow;
        }
      }]);

      return Util;
    }();

    Array.prototype.removeFromFront = function () {
      var item = null;
      this.reverse();
      item = this.pop();
      this.reverse();
      return item;
    };

    Array.prototype.copySortedArray = function () {
      var array = this.slice(0);
      array.sort();
      return array;
    };

    //=============================================================================
    // Public API                                                             
    //=============================================================================
    Object.assign($, {
      openInventory: function openInventory() {
        InventoryRequester.requestInventoryPageOpen(1);
      },
      openSmallInventory: function openSmallInventory() {
        InventoryRequester.requestSmallInventoryOpen();
      },
      closeInventory: function closeInventory() {
        InventoryRequester.requestInventoryClose();
      },
      closeSmallInventory: function closeSmallInventory() {
        InventoryRequester.requestSmallInventoryClose();
      }
    });
  };

  if (CLIImports.Cecilia) Setup();
})(Cecilia);